// XML Note Tools
// Jeffrey J Weimer
// SVN History: $Revision: 629 $ on $Date: 2008-03-25 19:40:10 -0400 (Tue, 25 Mar 2008) $

// Accessible Functions

// XMLPrint(property,text,[attributes, values, pretty])
// prints XML string to history
//	property: string of propert(ies)	- "property1:property2: ... propertyN"
//	text: string text to put at final property
//	attributes: string list of attributes		- "attribute1;attribute2; ... attributeN"
//	values: list of values for each attribute
//	pretty: variable 	- 0 (default) returns without tabs and returns, 1 returns with tabs and returns

// XMLPrintf(property,text,[attributes, values, pretty])
// same as above but returns string rather than prints to history

// XMLWaveNote(twave,property,text,[attributes, values, pretty, replace, at])
// same as above but attaches text to wave twave (full path or current folder only!)
//	replace: variable		- 0 (default), N => replace forward at N, -N => replace reverse at N
//	at:	string	"end" (default), "start"
//	for a note containing T total properties, replace=(-N) is the same as replace=(T-N+1)

// XMLNoteText(ntbk,property,text,[attributes, values, pretty, replace, at])
// same as above but puts text to notebook ntbk
//	at:	string	"end" (default), "start", "current"
//	selection always remains immediately after insertion for next addition!

// XMLNoteObject(ntbk,property,object,[objSpecs, attributes, values, pretty, at])
// same as above but puts an object (graph, table, ...) to notebook ntbk
//	objSpecs:	string list of four parameters (see Notebook ... frame={}, scale={}, picture={})
//	frame (default=0); scale (default=100); mode (default=0); flag (default=1); ;

// NT_LoadXMLTemplate(template,[filename])
// loads an XML file as a template (empty places contain #)

// NT_FillXMLTemplate(template,paramlist)
// returns text that is the template filled with paramlist in each #

// VARIATIONS
// text or values prefaced with % are interpreted as STRING commands to be executed
// to override this, use \% (escape code sequence)

#pragma rtGlobals=1

// PACKAGE GLOBAL FUNCTIONS

#pragma IgorVersion=6.03
#pragma version = 0.50
#include "XMLNoteToolsModule"
#include "XMLNoteToolsPanel"

// USER ACCESSIBLE FUNCTIONS

// Print in XML Format

Function XMLPrint(property,text,[attributes, values, pretty])
	string property, text, attributes, values
	variable pretty
	
	if (ParamIsDefault(pretty))
		pretty = 0
	endif
	
	if (ParamIsDefault(attributes))
		attributes = ""
		values = ""
	else
		if (ParamIsDefault(values))
			values = ""
		endif
	endif
	
	print xmlprintf(property, text, attributes=attributes, values=values, pretty=pretty)

	return 0
end

// Return an XML Formatted String

Function/S XMLPrintF(property,text,[attributes, values, pretty])
	string property, text, attributes, values
	variable pretty
	
	string theText
	
	if (ParamIsDefault(pretty))
		pretty = 0
	endif
	
	if (ParamIsDefault(attributes))
		attributes = ""
		values = ""
	else
		if (ParamIsDefault(values))
			values = ""
		endif
	endif
	
	theText = xmlnotetools#parseinput(property, text, pretty, attributes, values)

	return theText
end

// Put an XML Formatted String into a Wave Note

Function XMLWaveNote(twave,property,text,[attributes, values, pretty, replace, at])
	wave twave
	string property, text, attributes, values, at
	variable pretty, replace
	
	if (!WaveExists(twave))
		return -1
	endif

	if (ParamIsDefault(replace))
		replace = 0
	else
		pretty = 0
		at = ""
		attributes = ""
		values = ""
	endif
	
	if (ParamIsDefault(attributes))
		attributes = ""
		values = ""
	else
		if (ParamIsDefault(values))
			values = ""
		endif
	endif
	
	if (ParamIsDefault(pretty))
		pretty = 0
	endif
	
	if (ParamIsDefault(at))
		at = "end"
	endif
	
	return(xmlnotetools#wavenote(twave, property, text, replace, pretty, at, attributes, values))
end

// Put an XML Formatted String onto a Notebook

Function XMLNoteText(ntbk,property,text,[attributes, values, pretty, replace, at])
	string ntbk, property, text, attributes, values, at
	variable pretty, replace
	
	if (strlen(ntbk)==0)
		ntbk = WinName(0,16)
	else		
		if (strlen(WinList(ntbk,";","WIN:16"))==0)
			return -1
		endif
	endif

	if (ParamIsDefault(replace))
		replace = 0
	else
		pretty = 0
		at = ""
		attributes = ""
		values = ""
	endif
	
	if (ParamIsDefault(attributes))
		attributes = ""
		values = ""
	else
		if (ParamIsDefault(values))
			values = ""
		endif
	endif
	
	if (ParamIsDefault(pretty))
		pretty = 0
	endif
	
	if (ParamIsDefault(at))
		at = "current"
	endif
	
	return(xmlnotetools#notetext(ntbk, property, text, replace, pretty, at, attributes, values))

end

// Put an XML Formatted Object onto a Notebook

Function XMLNoteObject(ntbk,property,obj,[oSpecs,attributes, values, pretty, replace, at])
	string ntbk, property, obj, oSpecs, attributes, values, at
	variable pretty, replace

	variable rtn, ic
	
	if (strlen(ntbk)==0)
		ntbk = WinName(0,16)
	else		
		if (strlen(WinList(ntbk,";","WIN:16"))==0)
			return -1
		endif
	endif
	
	if (ParamIsDefault(replace))
		replace = 0
	endif

	if (ParamIsDefault(pretty))
		pretty = 0
	endif
	
	if (ParamIsDefault(at))
		at = "current"
	endif
	
	if (ParamIsDefault(attributes))
		attributes = ""
		values = ""
	else
		if (ParamIsDefault(values))
			values = ""
		endif
	endif
	
	if (ParamIsDefault(oSpecs))
		oSpecs = ""
	endif
	
	if (strlen(obj)==0)
		ic = 0
		for(ic=0;ic<9;ic+=1)
			obj = WinName(0,2^ic,1)
			if (strlen(obj)!=0)
				break
			endif
		endfor
	endif
	
	rtn = xmlnotetools#noteobject(ntbk,property,obj,replace,pretty,at,oSpecs,attributes,values)
	
	return rtn
end

// Load an XML Template into Storage

Function NT_LoadXMLTemplate(template,[filename])
	string template, filename
	
	variable rtn
	
//	rtn = xmlnotetools#loadtemplate(template,filename=filename)
	return rtn
end

// Fill an XML Template with Parameters

Function/S NT_FillXMLTemplate(template,paramlist)
	string template, paramlist
	
	string rtStr
	
//	rtStr = xmlnotetools#filltemplate(template,paramlist)
	
	return rtStr
end

// Display the XML Panel

Function ShowXMLPanel()

	STRUCT XMLNoteToolsPanelStruct xmlNT

	if (strlen(WinList("XMLNoteToolsPanel",";","WIN:64"))!=0)
		DoWindow/F XMLNoteToolsPanel
	else	
		PutUpXMLNTPanel(xmlNT)
	endif
	
	return 0
end
